"{ Package: 'jv:vdb' }"

TextCollector subclass:#VDBConsoleView
	instanceVariableNames:'debuggerHolder debugger'
	classVariableNames:'Prompt'
	poolDictionaries:''
	category:'VDB-UI-Console'
!

!VDBConsoleView class methodsFor:'initialization'!

initialize
    "Invoked at system start or when the class is dynamically loaded."

    "/ please change as required (and remove this comment)

    Prompt := '(gdb) '.

    "Modified: / 06-06-2014 / 22:45:13 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!VDBConsoleView methodsFor:'aspects'!

debuggerHolder
    "return/create the 'debuggerHolder' value holder (automatically generated)"

    debuggerHolder isNil ifTrue:[
        debuggerHolder := ValueHolder new.
        debuggerHolder addDependent:self.
    ].
    ^ debuggerHolder
!

debuggerHolder:aValueModel
    "set the 'debuggerHolder' value holder (automatically generated)"

    |oldValue newValue|

    debuggerHolder notNil ifTrue:[
        oldValue := debuggerHolder value.
        debuggerHolder removeDependent:self.
    ].
    debuggerHolder := aValueModel.
    debuggerHolder notNil ifTrue:[
        debuggerHolder addDependent:self.
    ].
    newValue := debuggerHolder value.
    oldValue ~~ newValue ifTrue:[
        self update:#value with:newValue from:debuggerHolder.
    ].
! !

!VDBConsoleView methodsFor:'change & update'!

update:something with:aParameter from:changedObject
    "Invoked when an object that I depend upon sends a change notification."

    changedObject == debuggerHolder ifTrue:[
        debuggerHolder value == debugger ifFalse:[
            debugger notNil ifTrue:[ 
                self unsubscribe.
            ].
            debugger := debuggerHolder value.
            debugger notNil ifTrue:[ 
                self subscribe.
            ].
        ].
        ^ self.
    ].
    super update:something with:aParameter from:changedObject

    "Modified: / 06-06-2014 / 22:10:04 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!VDBConsoleView methodsFor:'cursor handling'!

cursorMovementAllowed
    "return true, if the user may move the cursor around
     (via button-click, or cursor-key with selection).
     Here false is returned - the cursor is only moved by
     cursor positioning escape sequences arriving from the
     stream."

    ^ false

    "Created: / 07-06-2014 / 01:18:10 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!VDBConsoleView methodsFor:'event handling'!

keyPress: key x: x y: y

    self cursorLine < self list size ifTrue:[ 
        ^ self.
    ].

    key == #CursorLeft ifTrue:[ 
        (self cursorCol) > (Prompt size + 2) ifTrue:[
            ^ super keyPress: key x: x y: y    
        ].
        ^ self
    ].


    key == #CursorUp ifTrue:[ 
        self keyPressCursorUp.
        ^ self
    ].
    key == #CursorDown ifTrue:[ 
        self keyPressCursorDown.
        ^ self.
    ].
    key == #Return ifTrue:[ 
        self keyPressReturn.
        ^ super keyPress: key x: x y: y
    ].

    ^ super keyPress: key x: x y: y

    "Created: / 06-06-2014 / 22:33:47 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 07-06-2014 / 01:22:31 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

keyPressCursorDown

    "Created: / 06-06-2014 / 22:36:20 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

keyPressCursorUp

    "Created: / 06-06-2014 / 22:36:20 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

keyPressReturn
    | command |

    command := self list at: self cursorLine.
    command isEmptyOrNil ifTrue:[ ^ self ].
    command := command copyFrom: Prompt size + 1.
    command isEmptyOrNil ifTrue:[ 
        self showPrompt.
    ] ifFalse:[
        debugger send: (GDBCLICommand new value: command) wait: false.
    ].

    "Created: / 06-06-2014 / 22:36:21 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

onCommandEvent: event

    "Created: / 06-06-2014 / 22:43:45 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

onCommandResultEvent: event
    self showPrompt.

    "Created: / 06-06-2014 / 22:44:03 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

onConsoleOutputEvent: event
    self show: event value

    "Created: / 06-06-2014 / 21:45:16 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

onTargetOutputEvent: event
    self show: event value

    "Created: / 06-06-2014 / 21:45:22 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!VDBConsoleView methodsFor:'initialization & release'!

initialize
    super initialize.
    self showPrompt.

    "Created: / 06-06-2014 / 22:41:59 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

subscribe   
    "Register for debugger events. To be overrided by subclasses"

    debugger announcer 
        when: GDBCommandEvent           send: #onCommandEvent:          to: self;
        when: GDBCommandResultEvent     send: #onCommandResultEvent:    to: self;

        when: GDBConsoleOutputEvent     send: #onConsoleOutputEvent:    to: self;
        when: GDBTargetOutputEvent      send: #onTargetOutputEvent:     to: self.

    "Created: / 06-06-2014 / 21:26:48 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

unsubscribe
    "Unsubscribe myself fo debugger events"

    debugger announcer unsubscribe: self.
    debugger := nil.

    "Created: / 06-06-2014 / 21:26:48 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!VDBConsoleView methodsFor:'selection handling'!

autoMoveCursorToEndOfSelection
    "Redefined to return false since the cursor should
     not be affected by selecting"

    ^ false

    "Created: / 07-06-2014 / 01:20:49 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!VDBConsoleView methodsFor:'stream messages'!

showPrompt
    self nextPutAll: Prompt

    "Created: / 06-06-2014 / 22:41:59 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !


VDBConsoleView initialize!
